/*************************************************************************************
 *
 *  math_ext.h:  useful math functions
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#ifndef _JSMATH_MATH_EXT_H
#define _JSMATH_MATH_EXT_H

#define _USE_MATH_DEFINES
#include <math.h>

namespace Math
{
	class Functions
	{
	public:

		typedef double (*Integral)(double);

		static long double sin_angle(const long double angle);
		static long double cos_angle(const long double angle);
		static long double tan_angle(const long double angle);
		static long double asin_angle(const double x);
		static long double acos_angle(const double x);
		static long double atan_angle(const double x);
		static long double atan2_angle(const double y, const double x);

		static double Random(long idum = 1);

		static double TrapezoidalIntegration( Integral func, double lower, double upper, int steps );
	    static double SimpsonIntegration( Integral f, double lower,
            double upper, int steps);

		typedef double (*Function)(double);
		static double FindMinimum( double (*f)(double), double L, double U, double tol);
		static double FindMaximum( double (*f)(double), double L, double U, double tol);

		//  Error and Zeta Functions
		/*
		The error and Riemann zeta functions frequently occur in probability and statistics problems.
		ERF(z) is the integral of the Gaussian distribution from 0 to z.  
		*/
		static double erf( double q );

		//  n = number of points, x = values, f = function values, val = requested value
		static double LagrangeInterpolation(int n, double x[], double f[], double val);
		static void PolynomialInterpolation(double xa[], double ya[], int n, double x, double & y/*out*/, double & dy/*out*/);
		static void RationalInterpolation(double xa[], double ya[], int n, double x, double & y, double & dy);
	};

   	class AdditiveIntegral
	{
	public:

		AdditiveIntegral();
		double Integrate(Functions::Integral func, double upper, int steps);

	protected:

		double m_integral;
		double m_xi;
	};

}	//  end namespace Math

#endif // _JSMATH_MATH_EXT_H
