/*************************************************************************************
 *
 *  modelmatrix.h:  matrix operations
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#ifndef _JSMATH_MODELMATRIX_H
#define _JSMATH_MODELMATRIX_H

//  OpenGL works with modelview matrix -- this means, that modelling
//  and viewing matrix are combinated into this one.
//  This class is a transformation matrix, specialy designed for 
//  viewing transformations.

#include "matrix.h"
#include "quaternion.h"

namespace Math
{
	class ModelMatrix: public Matrix<double>
	{
	protected:

		CQuaternion	m_q;

		void Transform( CQuaternion & rot );

	public:

		ModelMatrix();
		ModelMatrix( const ModelMatrix & original );
		ModelMatrix( const CQuaternion & q );
		virtual ~ModelMatrix() {}
		ModelMatrix & operator=( const ModelMatrix & right_side );

		//  Like in an aeroplane
		//
		void ChangeHeading( double angle );	//  rotate around local "y", knodwn as "yaw" too
		void ChangePitch( double angle );	//  around "x"
		void ChangeRoll( double angle );	//  around "z"
        void ChangeRotation( Matrix<> & rotation );
        void ChangePosition( Math::Vector<> & position );

		//  With respect to cartesian coordinate system
		//

		//  gets
		Math::Vector<> GetZAxis();				//  get z axis
        Math::Vector<> GetYAxis();				//  y
        Math::Vector<> GetXAxis();				//  x

		CQuaternion GetQuaternion();

		double GetLambda();					//  get angle of rotation around y-axis
        double GetPhi();					//  x
        double GetTheta();					//  z

		Math::Vector<> GetPosition();

		//  sets
		void ChangeLambda( double angle );	//  rotate around y

		void SetLambda( double angle );
		void SetPhi( double angle );
		void SetTheta( double angle );

		void SetPosition( Math::Vector<> & position );

		ModelMatrix & operator=( const Matrix<> & right_side );
	};

}	//  end namespace Math

#endif  //  _JSMATH_MODELMATRIX_H
