/*************************************************************************************
 *
 *  direct.cpp:  direct N-body algorithm
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "../../inc/simulation/direct.h"

Direct::Direct(double G, double dt)
{
	m_G = G;
	m_dt = dt;
}

void Direct::SetLeaves( std::vector<JakubSchwarzmeier::Simulations::BarnesHut::Leaf*> * n_ptrLeavesList )
{
	m_ptrLeavesList = n_ptrLeavesList;
}

void Direct::Write(JakubSchwarzmeier::Simulations::BarnesHut::DataWriter * dw)
{
	for(size_t i = 0; i < m_ptrLeavesList->size(); i++)
	{
		(*m_ptrLeavesList)[i]->Write(dw);
	}
}

void Direct::Step()
{
	//  prepare acceleration for this step
	for(size_t i = 0; i < m_ptrLeavesList->size(); i++)
	{
		m_ptrLeavesList->at(i)->m_acceleration = 0;
	}

    for(size_t a = 0; a  < (m_ptrLeavesList->size()-1); a++)		// no self-interaction
		for (size_t b = a + 1; b < m_ptrLeavesList->size(); b++)
	{
		Math::Vector<> rvect = m_ptrLeavesList->at(b)->m_position - m_ptrLeavesList->at(a)->m_position;
        double d = rvect.GetLength();

        // force that acts (b)-->(a); accel to (a) from (b)
        m_ptrLeavesList->at(a)->m_acceleration += ( m_G * rvect * m_ptrLeavesList->at(b)->m_mass ) / pow(d, 3.0);
        m_ptrLeavesList->at(b)->m_acceleration += ( -m_G * rvect * m_ptrLeavesList->at(a)->m_mass ) / pow(d, 3.0);
    }

	//  integrate
    for(size_t i = 0; i < m_ptrLeavesList->size(); i++)
	{
		m_ptrLeavesList->at(i)->m_velocity += m_ptrLeavesList->at(i)->m_acceleration * m_dt;
		m_ptrLeavesList->at(i)->m_position += m_ptrLeavesList->at(i)->m_velocity * m_dt;
	}
}
