/////////////////////////////////////////////////////////////////////////////
// Name:        CamerasPanel.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     02/02/2007 10:22:22
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "CamerasPanel.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "CamerasPanel.h"
#include "cameradlg.h"
#include "digalexapp.h"
#include "digalexview.h"

////@begin XPM images
////@end XPM images

/*!
 * CCamerasPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CCamerasPanel, wxPanel )

/*!
 * CCamerasPanel event table definition
 */

BEGIN_EVENT_TABLE( CCamerasPanel, wxPanel )

////@begin CCamerasPanel event table entries
    EVT_BUTTON( XRCID("ID_ADDCAMERA"), CCamerasPanel::OnAddCamera )

    EVT_BUTTON( XRCID("ID_REMOVECAMERA"), CCamerasPanel::OnRemovecameraClick )
    EVT_UPDATE_UI( XRCID("ID_REMOVECAMERA"), CCamerasPanel::OnRemovecameraUpdate )

    EVT_TREE_ITEM_ACTIVATED( XRCID("ID_CAMERAS"), CCamerasPanel::OnCamerasItemActivated )

////@end CCamerasPanel event table entries

END_EVENT_TABLE()

/*!
 * CCamerasPanel constructors
 */

CCamerasPanel::CCamerasPanel( )
{
    Init();
}

CCamerasPanel::CCamerasPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}

/*!
 * CCamerasPanel creator
 */

bool CCamerasPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCamerasPanel creation
    SetParent(parent);
    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CCamerasPanel creation
    return true;
}

/*!
 * Member initialisation 
 */

void CCamerasPanel::Init()
{
////@begin CCamerasPanel member initialisation
    m_camerasCtrl = NULL;
////@end CCamerasPanel member initialisation
}
/*!
 * Control creation for CCamerasPanel
 */

void CCamerasPanel::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/cameraspanel.xrc"));

////@begin CCamerasPanel content construction
    if (!wxXmlResource::Get()->LoadPanel(this, GetParent(), _T("IDD_CAMERASPANEL")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    m_camerasCtrl = XRCCTRL(*this, "ID_CAMERAS", wxTreeCtrl);
////@end CCamerasPanel content construction

	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CDigalexView * view = wx_reinterpret_cast( CDigalexView*, childFrame->GetView() );
	view->GetCanvas()->SetObserver( ::wxGetApp().GetDocument()->GetVisualization()->GetObserver() );

    // Create custom windows not generated automatically here.
////@begin CCamerasPanel content initialisation
////@end CCamerasPanel content initialisation
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ADDCAMERA
 */

void CCamerasPanel::OnAddCamera( wxCommandEvent& event )
{
    CCameraDlg* window 
		= new CCameraDlg;

	window->SetClientData( NULL );
	window->Create(this);
    window->Show(true);

	::wxGetApp().GetDocument()->Modify( TRUE );
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_REMOVECAMERA
 */

class CChunkPtr: public wxTreeItemData
{
public:	
	CChunkPtr(CChunkDesc * pChunkDesc = NULL) { m_pChunkDesc = pChunkDesc; }
	CChunkDesc * m_pChunkDesc;
};

void CCamerasPanel::OnRemovecameraClick( wxCommandEvent& event )
{
	CChunkPtr* chunk = reinterpret_cast<CChunkPtr*>( m_camerasCtrl->GetItemData( m_camerasCtrl->GetSelection() ) );

	std::vector<CChunkDesc*> * chunks = ::wxGetApp().GetDocument()->GetVisualization()->GetAnimation()->GetChunks();
	std::vector<CChunkDesc*>::iterator it;
	for( it = chunks->begin(); it != chunks->end(); it++)
	{
		if ( (*it) == chunk->m_pChunkDesc )
		{
			delete (*it);
			chunks->erase( it );
			break;
		}
	}

	UpdateCameras();

	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_REMOVECAMERA
 */

void CCamerasPanel::OnRemovecameraUpdate( wxUpdateUIEvent& event )
{
	wxTreeItemId x = m_camerasCtrl->GetSelection();
	event.Enable( x.IsOk() && IsEnabled() );
}

/*!
 * wxEVT_COMMAND_TREE_ITEM_ACTIVATED event handler for ID_CAMERAS
 */



void CCamerasPanel::OnCamerasItemActivated( wxTreeEvent& event )
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CDigalexView * view = wx_reinterpret_cast( CDigalexView*, childFrame->GetView() );

	CChunkPtr* chunk = reinterpret_cast<CChunkPtr*>( m_camerasCtrl->GetItemData( m_camerasCtrl->GetSelection() ) );

    CCameraDlg* window 
		= new CCameraDlg;

	window->SetClientData( chunk->m_pChunkDesc );
	window->Create(this);
    window->Show(true);

	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	doc->Modify( TRUE );
}

/*!
 * Should we show tooltips?
 */

bool CCamerasPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCamerasPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCamerasPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCamerasPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCamerasPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCamerasPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCamerasPanel icon retrieval
}

void CCamerasPanel::UpdateCameras()
{
	m_camerasCtrl->Hide();
	m_camerasCtrl->DeleteAllItems();
	wxTreeItemId idRoot = m_camerasCtrl->AddRoot( _T("Cameras"), -1, -1, new CChunkPtr(NULL) );

	std::vector<CChunkDesc*> * chunks = ::wxGetApp().GetDocument()->GetVisualization()->GetAnimation()->GetChunks();
	std::vector<CChunkDesc*>::const_iterator it;
	int i = 1;
	for( it = chunks->begin(); it != chunks->end(); it++)
	{
        wxString name;
		name.sprintf( _T("Camera%d"), i );
		wxTreeItemId nId = m_camerasCtrl->AppendItem( idRoot, name.c_str(), -1, -1, new CChunkPtr( (*it) ) );
		i++;
	}

	m_camerasCtrl->Show();
}
