#include "StdAfx.h"
#include "chunkdesc.h"
#include <schwarzmeier/xmllib.h>

CChunkDesc::CChunkDesc(void)
{
	m_endCamera = new CCamera;
	m_endSnapshot = 0;
	m_durationTime = 0;
}

CChunkDesc::~CChunkDesc(void)
{
	delete m_endCamera;
}

CChunkDesc::CChunkDesc( const CChunkDesc & original )
{
	operator=( original );
}

CChunkDesc & CChunkDesc::operator=( const CChunkDesc & right_side )
{
	if ( &right_side == this )  return (*this);

	delete m_endCamera;

	m_endCamera 
		= dynamic_cast<CCamera*>( right_side.m_endCamera->GetClassInfo()->CreateObject() );

	m_endCamera->Assign( right_side.m_endCamera );

	m_endSnapshot = right_side.m_endSnapshot;
	m_durationTime = right_side.m_durationTime;	

	return (*this);
}

double CChunkDesc::GetDuration()
{
	return m_durationTime;
}

void CChunkDesc::SetDuration(double durationTime)
{
	m_durationTime = durationTime;
}

long CChunkDesc::GetEndSnapshot()
{
	return m_endSnapshot;
}

void CChunkDesc::SetEndSnapshot(long snapShot)
{
	m_endSnapshot = snapShot;
}

CCamera * CChunkDesc::GetEndCamera()
{
	return m_endCamera;
}

void CChunkDesc::SetEndCamera( CCamera * endMatrix )
{
	delete m_endCamera;

	m_endCamera 
		= dynamic_cast<CCamera*>( endMatrix->GetClassInfo()->CreateObject() );

	m_endCamera->Assign( endMatrix );
}

xmlNode * CChunkDesc::Serialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	xmlNode * pChunkNode =  xmlNewNode( NULL, (const xmlChar *)s_strCameraChunk.c_str() );
	xmlAddChild( pXmlNode, pChunkNode );

	//  snapshot
	xmlNode * snapshotNode = xmlNewNode( NULL, (const xmlChar *)s_strSnapshot.c_str() );
	xmlAddChild( pChunkNode, snapshotNode );
	SetTextNode( pXmlDoc, snapshotNode, m_endSnapshot );

	//  durationtime
	xmlNode * durationtimeNode = xmlNewNode( NULL, (const xmlChar *)s_strDurationTime.c_str() );
	xmlAddChild( pChunkNode, durationtimeNode );
	SetTextNode( pXmlDoc, durationtimeNode, m_durationTime );

	//  endmatrix
	xmlNode * endMatrixNode = xmlNewNode( NULL, (const xmlChar *)s_strEndCamera.c_str() );
	xmlAddChild( pChunkNode, endMatrixNode );
	m_endCamera->Serialize( pXmlDoc, endMatrixNode );

	return pChunkNode;
}

void CChunkDesc::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode )
{
	std::string xpathSnapshotStr(s_strSnapshot);
	xpathSnapshotStr += "/text()";
	std::string end = GetXPathResponse( pXmlDoc, pXmlNode, xpathSnapshotStr );
	m_endSnapshot = atoi( end.c_str() );

	std::string xpathDurationStr(s_strDurationTime);
	xpathDurationStr += "/text()";
	std::string durat = GetXPathResponse( pXmlDoc, pXmlNode, xpathDurationStr );
	m_durationTime = atof( durat.c_str() );

	xmlNode * endmatNode = SelectSingleNode( pXmlDoc, pXmlNode, s_strEndCamera );
	delete m_endCamera;
	m_endCamera = CCamera::Deserialize( endmatNode ) ;
	m_endCamera->Deserialize( pXmlDoc, endmatNode );
}

std::string CChunkDesc::s_strCameraChunk( _T("camerachunk") );
std::string CChunkDesc::s_strSnapshot( _T("endsnapshot") );
std::string CChunkDesc::s_strDurationTime( _T("durationtime") );
std::string CChunkDesc::s_strEndCamera( _T("endmatrix") );
