#include "StdAfx.h"

#include "stararchive.h"
#include <schwarzmeier/xmllib.h>

//  LIBXML
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

#include <crtdbg.h>

CStarArchive::CStarArchive(void)
{
}

CStarArchive::~CStarArchive(void)
{
	LIST::const_iterator it;
	for(it = m_leaves.begin(); it != m_leaves.end(); it++ )
	{
		delete *it;
	}
}

void CStarArchive::Open( wxString filename )
{
	m_filename = filename;
	ReRead();
}

CStarArchive::LIST * CStarArchive::GetList()
{
	return &m_leaves;
}

void CStarArchive::ReRead()
{
	//  Read parameters from XML file
	xmlDoc * pXmlDoc = xmlParseFile( m_filename );
	xmlNode * pRootNode = xmlDocGetRootElement(pXmlDoc);

	//  read count
	xmlNodeSet * pSnapshotNodes = SelectNodes( pXmlDoc, pRootNode, "./snapshot" );
	m_nSnapShots = pSnapshotNodes->nodeNr;

	//  read filename
	std::string strFn = GetXPathResponse( pXmlDoc, pRootNode, "./filename/text()" );
	m_binFileName = strFn.c_str();

	xmlFreeDoc( pXmlDoc );
    xmlCleanupParser(); 
}

CStarArchive::LIST * CStarArchive::GetSnapShot(int no)
{
	if (no < m_nSnapShots)
	{
		// 
		xmlDoc * pXmlDoc = xmlParseFile( m_filename );
		xmlNode * pRootNode = xmlDocGetRootElement(pXmlDoc);

		//  read count
		wxString strXPathCount;

		strXPathCount.sprintf("./snapshot[@id=%d]", no);

		xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
		context->node = pRootNode;
		xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPathCount.c_str(), context);
		if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
		{
		//	return -1;
		}
		xmlXPathFreeContext(context);

		long long seekPos;
		GetAttribute( xpath->nodesetval->nodeTab[0], "start", &seekPos );

		///

			//  read count
		strXPathCount.sprintf("./snapshot[@id=%d]/count/text()", no);

		context = xmlXPathNewContext(pXmlDoc);
		context->node = pRootNode;
		xpath = xmlXPathEvalExpression( (const xmlChar *)strXPathCount.c_str(), context);
		if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
		{
			//return -1;
		}
		xmlXPathFreeContext(context);

		xmlChar * textValue = xmlNodeListGetString(pXmlDoc, xpath->nodesetval->nodeTab[0], 1);
		int count = atoi( (const char *)textValue );
		xmlFree(textValue);

		xmlFreeDoc( pXmlDoc );
	    xmlCleanupParser(); 

		//  Read stars from binary file
		//

		//  clean old list
		LIST::const_iterator it;
		for(it = m_leaves.begin(); it != m_leaves.end(); it++ )
		{
			delete *it;
		}
		m_leaves.clear();
		m_uniqueMass.clear();

		//  input/output binary stream
		JakubSchwarzmeier::Simulations::BarnesHut::fstream binIn;
		binIn.open(m_binFileName, std::ios_base::in | std::ios_base::binary );

		binIn.seekg( seekPos, std::ios_base::beg );

		//  memleak when reusing fstream in DataReader
		/*
	60 bytes long.
	Data: <N       E     > 94 A5 4E 10 08 00 00 00 18 10 45 00 00 00 00 00 
		*/
		JakubSchwarzmeier::Simulations::BarnesHut::DataReader dr;
		dr.OpenStream( &binIn );

		bool bMassEmpty = m_uniqueMass.empty();

		for(int i = 0; i < count; i++)
		{
			//  create leaf and inserts it between other leaves
			CBody * leaf = new CBody;
			leaf->Read(&dr);
			
			if (bMassEmpty)  
				m_uniqueMass.insert( leaf->m_mass );

			m_leaves.push_back( leaf );
		}

		return &m_leaves;
	}

	return NULL;
}

int CStarArchive::GetCount()
{
	return m_nSnapShots;
}

wxString CStarArchive::GetFileName()
{
	return m_filename;
}

CStarArchive::SET_MASSES * CStarArchive::GetMasses()
{
	return &m_uniqueMass;
}

CStarArchive::MAP_MASSES * CStarArchive::GetMassesMap()
{
	return &m_mapMass;
}

void CStarArchive::LockReading()
{
	m_csRead.Enter();
}

void CStarArchive::ReleaseReading()
{
	m_csRead.Leave();
}
