// RecognitaView.cpp : implementation of the CThreedCanvas class
//

#include "stdafx.h"
#include "DigalexApp.h"

#include "DigalexDoc.h"
#include "ThreedCanvas.h"

#include "DigalexView.h"

#include <strstream>
#include <time.h>

IMPLEMENT_DYNAMIC_CLASS(CThreedCanvas, JakubSchwarzmeier::wx::COpenGLCanvas)


// CThreedCanvas

// CThreedCanvas construction/destruction


CThreedCanvas::CThreedCanvas(wxWindow *parent)
{
	Create( parent );
}

void CThreedCanvas::Create(wxWindow *parent, bool bDeserialize)
{
	m_pHeadingFont = NULL;
	m_pBoldFont = NULL;
	m_bUseVideoDesc = false;
	m_bUseTime = false;
	m_bUseGrid = false;
	m_bUseCameraParameters = false;
	m_tCoeff = 0.0;

	COpenGLCanvas::Create(parent, bDeserialize);

	SetSignaled( false );
	m_bTexturing = false;
}

CThreedCanvas::~CThreedCanvas()
{
	if (m_pHeadingFont != NULL)
	{
		throw;
	}
	if (m_pBoldFont != NULL)
	{
		throw;
	}
}

//virtual
void CThreedCanvas::OnRender()
{
	//  clear the color and depth buffers 
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//  modeling transformation in ModelView matrix

	glLoadIdentity();

	//  simulate viewing transformation in ModelView matrix
	glMultMatrixd( m_observer->GetCamera()->GetMatrix() );

	//  objects drawing

	// We don't need to calculate this every frame, only when the camera view changes.
	// I just did it every frame anyway.  In this case it isn't a big deal.
    m_frustum.CreateFrustum();		//  calculate frustum 

	//  get stars
	CStarArchive * pStarArchive = ::wxGetApp().GetDocument()->GetStarArchive();
	pStarArchive->LockReading();
	CStarArchive::LIST * pList =  pStarArchive->GetList();

	//  frustum culling

	CStarArchive::LIST::const_iterator it;
	for(it = pList->begin(); it != pList->end(); it++ )
	{
		if ( m_frustum.PointInFrustum( (*it)->m_position ) )
		{
			glPushMatrix();		//  push current viewing matrix

			//  draw
			glBegin (GL_POINTS);

//				s_colors[2][4][2][3]
				
				CStarArchive::MAP_MASSES * pMassesMap = 
					pStarArchive->GetMassesMap();

				double mass = (*it)->m_mass;

				CStarArchive::MAP_MASSES::const_iterator itMass = 
					pMassesMap->find( mass );

				int i, j;

				if ( itMass == pMassesMap->end() )
				{
					i = 0;
					j = 0;
				}
				else
				{
					switch (itMass->second)
					{
					case 0:
						
						i = 0;
						j = 0;
						break;

					case 1:
						
						i = 0;
						j = 1;
						break;

					case 2:
						
						i = 0;
						j = 2;
						break;

					case 3:
						
						i = 0;
						j = 3;
						break;
					
					//
					case 4:
						
						i = 1;
						j = 0;
						break;

					case 5:
						
						i = 1;
						j = 1;
						break;

					case 6:
						
						i = 1;
						j = 2;
						break;

					case 7:
						
						i = 1;
						j = 3;
						break;
					}
				}

				int k = rand()%2;	//  <0;1>

				glColor3ub( s_colors[i][j][k][0],
							s_colors[i][j][k][1],
							s_colors[i][j][k][2] );

				glVertex3f( (*it)->m_position[0],
							(*it)->m_position[1],
							(*it)->m_position[2] );

			glEnd();

			glPopMatrix();		//  pop back current viewing matrix
		}
	}

	pStarArchive->ReleaseReading();

	Draw3DGrid();

	glLoadIdentity();
	DrawCameraParameters();
	DrawOSD();

	//  flush rendering pipe
	glFinish();	
}

//virtual
void CThreedCanvas::PreRender()
{
	COpenGLCanvas::PreRender();

	glEnable( GL_POINT_SMOOTH );

	wxFileName fn( ::wxGetApp().argv[0] );
	fn.SetFullName( _T("euclidb.ttf") );

	//  create font for OSD
	if (m_pHeadingFont != NULL)  throw;
	m_pHeadingFont = new JakubSchwarzmeier::wx::CFreetype;
	m_pHeadingFont->Create( fn.GetFullPath().c_str(), 13 );

	fn.SetFullName( _T("courbd.ttf") );
	if (m_pBoldFont != NULL)  throw;
	m_pBoldFont = new JakubSchwarzmeier::wx::CFreetype;
	m_pBoldFont->Create( fn.GetFullPath().c_str(), 13 );
}

// virtual
void CThreedCanvas::OnContextTerminate()
{
	COpenGLCanvas::OnContextTerminate();

	if (m_pHeadingFont != NULL)
	{
		delete m_pHeadingFont;
		m_pHeadingFont = NULL;
	}

	if (m_pBoldFont != NULL)
	{
		delete m_pBoldFont;
		m_pBoldFont = NULL;
	}
}

bool CThreedCanvas::OnSerialize( JakubSchwarzmeier::wx::CSerializeReaderWriter * dir )
{
	COpenGLCanvas::OnSerialize( dir );

	return true;
}

void CThreedCanvas::RenderFinished()
{
	COpenGLCanvas::RenderFinished();
}

void CThreedCanvas::SetTitle( std::string title )
{
	m_strTitle = title;
}

void CThreedCanvas::SetAffilation( std::string affilation )
{
	m_strAffilation = affilation;
}

void CThreedCanvas::SetAuthor( std::string author )
{
	m_strAuthor = author;
}

void CThreedCanvas::SetUseVideoDesc( bool bUseVideoDesc )
{
	m_bUseVideoDesc = bUseVideoDesc;
}

void CThreedCanvas::SetUseTime( bool bUseTime )
{
	m_bUseTime = bUseTime;
}

void CThreedCanvas::SetTimeCoefficient( double tCoeff )
{
	m_tCoeff = tCoeff;
}

void CThreedCanvas::SetUseGrid( bool bUseGrid )
{
	m_bUseGrid = bUseGrid;
}

void CThreedCanvas::SetUseCameraParameters( bool bUseCameraParameters )
{
	m_bUseCameraParameters = bUseCameraParameters;
}

void CThreedCanvas::Draw3DGrid()
{
	if (m_bUseGrid)
	{
		// Turn the lines GREEN
		glColor3ub(0, 255, 0);

		// Draw a 1x1 grid along the X and Z axis'
		for(float i = -50; i <= 50; i += 1)
		{
			// Start drawing some lines
			glBegin(GL_LINES);

				// Do the horizontal lines (along the X)
				glVertex3f(-50, 0, i);
				glVertex3f(50, 0, i);

				// Do the vertical lines (along the Z)
				glVertex3f(i, 0, -50);
				glVertex3f(i, 0, 50);

			// Stop drawing lines
			glEnd();
		}
	}
} 

void CThreedCanvas::DrawOSD()
{
	//  render labels
	//

	double top = 20.0;
	double side = 10.0;

	if (m_bUseTime)
	{
		//  left
		//
		glColor3ub(255, 255, 255);

		double time = m_tCoeff * ::wxGetApp().GetMainFrame()->GetSnapshot();
		wxString strTime;
		wxString strUnit;

		if ( time >= 1e9 )
		{
			time /= 1e9;
			strUnit = "Gyr";
		}
		else if ( time >= 1e6 )
		{
			time /= 1e6;
			strUnit = "Myr";
		}
		else
		{
			time /= 1e3;
			strUnit = "kyr";
		}
		
		strTime.sprintf( "Time = %.4g %s", time, strUnit );

		glColor3ub( 255, 255, 255 );
		m_pHeadingFont->PrintWindow( side, top, strTime.c_str(), 
			JakubSchwarzmeier::wx::CFreetype::TOP_RIGHT );

		//
		//  end left
	}

	if (m_bUseVideoDesc)
	{
		glColor3ub( 255, 255, 255 );
		m_pBoldFont->PrintWindow( side, top, m_strTitle, 
			JakubSchwarzmeier::wx::CFreetype::TOP_LEFT );

		m_pHeadingFont->PrintWindow( side, top, m_strAffilation, 
			JakubSchwarzmeier::wx::CFreetype::BOTTOM_LEFT );

		m_pHeadingFont->PrintWindow( side, top, m_strAuthor, 
			JakubSchwarzmeier::wx::CFreetype::BOTTOM_RIGHT );
	}
}

void CThreedCanvas::DrawCameraParameters()
{
	if (m_bUseCameraParameters)
	{
		int right = 10;
		int y = 48;

		JakubSchwarzmeier::Std::Graphics::RotationMatrix * cam = GetObserver()->GetCamera();
		JakubSchwarzmeier::Std::Graphics::RotationMatrix mat = *cam;
		mat.Inverse();

		char* u[3] = { "", "", "" }; //  units fill-in

		//  prepare printing string
		wxString camText;
		camText.sprintf("px: %.18g %s\npy: %.18g %s\npz: %.18g %s\nX\nex: %.18g\ney: %.18g\nez: %.18g\nY\nex: %.18g\ney: %.18g\nez: %.18g\nZ\nex: %.18g\ney: %.18g\nez: %.18g\nFOV: %.2f\u00b0",
			mat[12], u[0], mat[13], u[1], mat[14], u[2],
			(*cam)[0], (*cam)[1], (*cam)[2], 
			(*cam)[4], (*cam)[5], (*cam)[6], 
			(*cam)[8], (*cam)[9], (*cam)[10],
			GetFov()
			);

		glColor3ub( 255, 255, 255 );
		m_pHeadingFont->PrintWindow( right, y, camText.c_str(), JakubSchwarzmeier::wx::CFreetype::TOP_RIGHT );
	}
}

const double CThreedCanvas::s_colors[2][4][2][3] =
{
	{
		{
			{ 125, 190, 255 },	//  disk1, color1
			{ 255, 255, 255 }	//  disk1, color2
		},
		{
			{ 255, 211, 92 },	//  bulge1, color1
			{ 255, 239, 245 }	//  bulge1, color2
		},
		{
			{ 255, 128, 128 },	//  stellar halo1, color 1
			{ 255, 255, 255 }	//  stellar halo1, color 2
		},
		{
			{ 128, 0, 128 },	//  dark halo 1, color 1
			{ 157, 0, 157 }		//  dark halo 1, color 2
		}
	},
	{
		{
			{ 0, 128, 255 },	//  disk2, color1
			{ 0, 128, 255 }		//  disk2, color2
		},
		{
			{ 255, 255, 0 },	//  bulge2, color1
			{ 255, 255, 0 }		//  bulge2, color2	
		},
		{
			{ 255, 0, 0 },	//  stellar halo2, color 1
			{ 255, 255, 255 }	//  stellar halo2, color 2
		},
		{
			{ 128, 0, 255 },	//  dark halo 1, color 1
			{ 128, 0, 255 }	//  dark halo 2, color 2
		}
	}
};
