/////////////////////////////////////////////////////////////////////////////
// Name:        VideoMakerDlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     09/02/06 17:06:11
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "VideoMakerDlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "VideoMakerDlg.h"
#include "Visualization.h"
#include "DigalexApp.h"

#include "animation.h"

////@begin XPM images
////@end XPM images

/*!
 * CVideoMakerDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CVideoMakerDlg, wxDialog )

/*!
 * CVideoMakerDlg event table definition
 */

BEGIN_EVENT_TABLE( CVideoMakerDlg, wxDialog )

////@begin CVideoMakerDlg event table entries
    EVT_UPDATE_UI( XRCID("ID_WIDTH"), CVideoMakerDlg::OnCustomSizeUpdate )

    EVT_UPDATE_UI( XRCID("ID_HEIGHT"), CVideoMakerDlg::OnCustomSizeUpdate )

    EVT_UPDATE_UI( XRCID("ID_MAINTAINASPECT"), CVideoMakerDlg::OnCustomSizeUpdate )

    EVT_UPDATE_UI( XRCID("ID_X"), CVideoMakerDlg::OnAspectUpdate )

    EVT_UPDATE_UI( XRCID("ID_Y"), CVideoMakerDlg::OnAspectUpdate )

    EVT_BUTTON( wxID_OK, CVideoMakerDlg::OnOkClick )

////@end CVideoMakerDlg event table entries

END_EVENT_TABLE()

/*!
 * CVideoMakerDlg constructors
 */

CVideoMakerDlg::CVideoMakerDlg( )
{
}

CVideoMakerDlg::CVideoMakerDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CVideoMakerDlg creator
 */

bool CVideoMakerDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVideoMakerDlg member initialisation
    m_width = 0;
    m_height = 0;
    m_X = 0;
    m_Y = 0;
    m_800600Ctrl = NULL;
    m_1024768Ctrl = NULL;
    m_12801024Ctrl = NULL;
    m_16001200Ctrl = NULL;
    m_customSizeCtrl = NULL;
    m_maintainAspectCtrl = NULL;
////@end CVideoMakerDlg member initialisation

////@begin CVideoMakerDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CVideoMakerDlg creation
    return true;
}

/*!
 * Control creation for CVideoMakerDlg
 */

void CVideoMakerDlg::CreateControls()
{    

	wxXmlResource::Get()->Load(wxT("xrc/videomaker.xrc")); 
////@begin CVideoMakerDlg content construction
    if (!wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_VIDEOMAKER")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    m_800600Ctrl = XRCCTRL(*this, "ID_800x600", wxRadioButton);
    m_1024768Ctrl = XRCCTRL(*this, "ID_1024x768", wxRadioButton);
    m_12801024Ctrl = XRCCTRL(*this, "ID_1280x1024", wxRadioButton);
    m_16001200Ctrl = XRCCTRL(*this, "ID_1600x1200", wxRadioButton);
    m_customSizeCtrl = XRCCTRL(*this, "ID_CUSTOMSIZE", wxRadioButton);
    m_maintainAspectCtrl = XRCCTRL(*this, "ID_MAINTAINASPECT", wxCheckBox);
    // Set validators
    if (FindWindow(XRCID("ID_WIDTH")))
        FindWindow(XRCID("ID_WIDTH"))->SetValidator( wxIntegerValidator( & m_width ) );
    if (FindWindow(XRCID("ID_HEIGHT")))
        FindWindow(XRCID("ID_HEIGHT"))->SetValidator( wxIntegerValidator( & m_height ) );
    if (FindWindow(XRCID("ID_X")))
        FindWindow(XRCID("ID_X"))->SetValidator( wxIntegerValidator( & m_X ) );
    if (FindWindow(XRCID("ID_Y")))
        FindWindow(XRCID("ID_Y"))->SetValidator( wxIntegerValidator( & m_Y ) );
    if (FindWindow(XRCID("ID_FPS")))
        FindWindow(XRCID("ID_FPS"))->SetValidator( wxDoubleValidator( & m_fps ) );
    if (FindWindow(XRCID("ID_OUTPUTFN")))
        FindWindow(XRCID("ID_OUTPUTFN"))->SetValidator( wxNotNullValidator(& m_videoFile, _T("Please enter the name of video file.") ) );
////@end CVideoMakerDlg content construction

    // Create custom windows not generated automatically here.
	CVisualization * vis = reinterpret_cast<CVisualization*>( GetClientData() );

	m_videoFile = vis->GetAnimation()->GetOutputVideo().c_str();
	if ( m_videoFile.IsEmpty() )
	{
		wxFileName newFN( ::wxGetApp().GetDocument()->GetStarArchive()->GetFileName() );
		newFN.SetExt( _T("avi") );
		newFN.MakeAbsolute();
		m_videoFile = newFN.GetFullPath();
	}

	m_fps = vis->GetAnimation()->GetFps();

////@begin CVideoMakerDlg content initialisation
////@end CVideoMakerDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CVideoMakerDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVideoMakerDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVideoMakerDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVideoMakerDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVideoMakerDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVideoMakerDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVideoMakerDlg icon retrieval
}
/*!
 * wxEVT_UPDATE_UI event handler for ID_WIDTH
 */

void CVideoMakerDlg::OnCustomSizeUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_customSizeCtrl->GetValue() );
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_X
 */

void CVideoMakerDlg::OnAspectUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_customSizeCtrl->GetValue() && m_maintainAspectCtrl->GetValue() );
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CVideoMakerDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CVisualization * vis = reinterpret_cast<CVisualization*>( GetClientData() );

		//  set output video
		wxFileName fn( m_videoFile );
		fn.MakeAbsolute();

		if ( wxFileName::FileExists( fn.GetFullPath() ) )
		{
			remove( fn.GetFullPath().c_str() );
		}

		fn.SetExt( "avi" );

		vis->GetAnimation()->SetOutputVideo( fn.GetFullPath().c_str() );

		//  set fps
		vis->GetAnimation()->SetFps( m_fps );

		//  get size
		if ( m_customSizeCtrl->GetValue() )
		{
            if ( m_maintainAspectCtrl->GetValue() )
			{
                m_height = m_width * (double)((double)m_Y/(double)m_X);
			}
		}
		else
		{
            if ( m_800600Ctrl->GetValue() )
			{
				m_width = 800;
				m_height = 600;
			}
			else
            if ( m_1024768Ctrl->GetValue() )
			{
				m_width = 1024;
				m_height = 768;
			}
			else
            if ( m_12801024Ctrl->GetValue() )
			{
				m_width = 1280;
				m_height = 1024;
			}
			else
            if ( m_16001200Ctrl->GetValue() )
			{
				m_width = 1600;
				m_height = 1200;
			}
		}

		//  generate video
		size_t steps = vis->GetAnimation()->Create( vis, m_fps, m_width, m_height );

		// perform animation
		bool cont = true;

		//  progress dialog
		wxProgressDialog dialog(_T("Animation Run"), 
			_T("Performing visualization and encoding video..."), steps, this, 
			wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT |  wxPD_ELAPSED_TIME |
            wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME | wxPD_SMOOTH );

		for(size_t i = 0; i < steps; i++)
		{
			//  actual animation writing
            vis->GetAnimation()->Update();

			//  progress dialog stuff
	        if ( fmod( i, (double)steps/100.0 ) == 0 ) // only 100 updates, this makes it much faster
				cont = dialog.Update(i);

			if ( !cont )
			{
				if ( wxMessageBox(_T("Do you really want to cancel?"),
								_T("Progress dialog question"),  // caption
								wxYES_NO | wxICON_QUESTION) == wxYES )
				{
					break;
				}
				else
				{
					cont = true;
					dialog.Resume();
				}
			}
		}

		if (cont) 
		{
			vis->GetAnimation()->Finish();

			wxMessageBox(_T("Animation done!") );
			dialog.Destroy();
		}

		EndDialog( wxID_OK );
	}
}


