#include "StdAfx.h"
#include "Visualization.h"
#include "digalexapp.h"
#include "digalexview.h"

CVisualization::CVisualization(void)
{
	m_pAnimation = new CAnimation;

	m_pObserver = new CGalaxyObserver;
	m_pObserver->OnCreateCamera();

	m_pOpenGlCanvas = NULL;
}

CVisualization::~CVisualization(void)
{
	delete m_pAnimation;
	m_pObserver->Destroy();
}

CAnimation * CVisualization::GetAnimation()
{
	return m_pAnimation;
}

CGalaxyObserver * CVisualization::GetObserver()
{
	return m_pObserver;
}

void CVisualization::Serialize(xmlDoc * pXmlDoc, xmlNode * branch)
{
	xmlNode * visualizationNode = SelectSingleNode( pXmlDoc, branch, s_strVisualization );
	if (visualizationNode != NULL)
	{
		//  remove old visualization node
		xmlUnlinkNode( visualizationNode );
		xmlFreeNode( visualizationNode );
	}

	//  simulation
	xmlNode * pVisualizationNode =  xmlNewNode( NULL, (const xmlChar *)s_strVisualization.c_str() );
	xmlAddChild( branch, pVisualizationNode );

	//  serialize animation
	m_pAnimation->Serialize( pXmlDoc, pVisualizationNode );
}

void CVisualization::Deserialize(xmlDoc * pXmlDoc, xmlNode * pXmlNode)
{
	xmlNode * visualizationNode = SelectSingleNode( pXmlDoc, pXmlNode, s_strVisualization );
	if (visualizationNode == NULL)  return;
	
	//  deserialize animation
	m_pAnimation->Deserialize( pXmlDoc, visualizationNode );
}

CThreedCanvas * CVisualization::GetThreedCanvas()
{
	if (m_pOpenGlCanvas == NULL)
	{
		wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, ::wxGetApp().GetMainFrame()->GetActiveChild() );
		CDigalexView * view = wx_reinterpret_cast( CDigalexView*, childFrame->GetView() );

		m_pOpenGlCanvas = wx_reinterpret_cast( CThreedCanvas*, view->GetCanvas() );
	}

	return m_pOpenGlCanvas;
}

std::string CVisualization::s_strVisualization( _T("visualization") );
