/////////////////////////////////////////////////////////////////////////////
// Name:        cameradlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     05/02/2007 11:55:04
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "cameradlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "cameradlg.h"
#include "cameraspanel.h"
#include "digalexapp.h"
#include "digalexview.h"
#include "Camera.h"
#include <schwarzmeier/std.h>
#include "GalaxyObserver.h"

////@begin XPM images
////@end XPM images

/*!
 * CCameraDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CCameraDlg, wxDialog )

/*!
 * CCameraDlg event table definition
 */

BEGIN_EVENT_TABLE( CCameraDlg, wxDialog )

////@begin CCameraDlg event table entries
    EVT_PAINT( CCameraDlg::OnPaint )

    EVT_BUTTON( XRCID("ID_GETSTARTING"), CCameraDlg::OnGetstartingClick )
    EVT_UPDATE_UI( XRCID("ID_GETSTARTING"), CCameraDlg::OnUpdateUI )

    EVT_BUTTON( XRCID("ID_APPLYSTARTING"), CCameraDlg::OnApplystartingClick )
    EVT_UPDATE_UI( XRCID("ID_APPLYSTARTING"), CCameraDlg::OnUpdateUI )

    EVT_BUTTON( XRCID("ID_GETENDING"), CCameraDlg::OnGetendingClick )

    EVT_BUTTON( XRCID("ID_APPLYENDING"), CCameraDlg::OnApplyendingClick )

    EVT_BUTTON( wxID_OK, CCameraDlg::OnOkClick )

////@end CCameraDlg event table entries

END_EVENT_TABLE()

/*!
 * CCameraDlg constructors
 */

CCameraDlg::CCameraDlg( )
{
    Init();
}

CCameraDlg::CCameraDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CCameraDlg creator
 */

bool CCameraDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCameraDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CCameraDlg creation
    return true;
}

/*!
 * Member initialisation 
 */

void CCameraDlg::Init()
{
////@begin CCameraDlg member initialisation
////@end CCameraDlg member initialisation
}
/*!
 * Control creation for CCameraDlg
 */

void CCameraDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/cameradlg.xrc"));

////@begin CCameraDlg content construction
    if (!wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("IDD_ORBITERCAMERA")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    // Set validators
    if (FindWindow(XRCID("ID_STARTINGMATRIX")))
        FindWindow(XRCID("ID_STARTINGMATRIX"))->SetValidator( wxGenericValidator(& m_startMatrix) );
    if (FindWindow(XRCID("ID_STARTINGTIME")))
        FindWindow(XRCID("ID_STARTINGTIME"))->SetValidator( wxIntegerValidator( & m_startSnapshot) );
    if (FindWindow(XRCID("ID_ENDINGMATRIX")))
        FindWindow(XRCID("ID_ENDINGMATRIX"))->SetValidator( wxGenericValidator(& m_endMatrix) );
    if (FindWindow(XRCID("ID_ENDINGTIME")))
        FindWindow(XRCID("ID_ENDINGTIME"))->SetValidator( wxIntegerValidator( & m_endSnapshot) );
    if (FindWindow(XRCID("ID_ANIMATIONLENGTH")))
        FindWindow(XRCID("ID_ANIMATIONLENGTH"))->SetValidator( wxDoubleValidator(& m_animationDuration) );
////@end CCameraDlg content construction

	GetParent()->Enable( false );

	CDigalexDoc * doc = ::wxGetApp().GetDocument();

	if ( GetClientData() == NULL )
	{
		m_localChunkDesc = new CChunkDesc;
		::wxGetApp().GetDocument()->GetVisualization()->GetAnimation()->GetChunks()->push_back( m_localChunkDesc );

		doc->GetVisualization()->GetAnimation()->SetStartCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );
		m_localChunkDesc->SetEndCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );

		reinterpret_cast<CCamerasPanel*>( GetParent() )->UpdateCameras();
	}
	else
	{
		m_localChunkDesc = reinterpret_cast<CChunkDesc*>( GetClientData() );
	}	

	m_animationDuration = m_localChunkDesc->GetDuration(); 
	
	JakubSchwarzmeier::Std::Graphics::RotationMatrix mat;
	JakubSchwarzmeier::Std::Math::Vector<> vec;

	mat = *( doc->GetVisualization()->GetAnimation()->GetStartCamera() );
	vec = mat.GetPosition();
	m_startMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	mat = *( m_localChunkDesc->GetEndCamera() );
	vec = mat.GetPosition();
	m_endMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	m_startSnapshot = doc->GetVisualization()->GetAnimation()->GetStartSnapshot();
	m_endSnapshot = m_localChunkDesc->GetEndSnapshot();

    // Create custom windows not generated automatically here.
////@begin CCameraDlg content initialisation
////@end CCameraDlg content initialisation
}

/*!
 * wxEVT_PAINT event handler for IDD_ORBITERCAMERA
 */

void CCameraDlg::OnPaint( wxPaintEvent& event )
{
    wxPaintDC dc(this);

	if (GetSizer())
	{
		GetSizer()->Fit(this);
		GetSizer()->SetSizeHints(this);
		Layout();
	}
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GETSTARTING
 */

void CCameraDlg::OnGetstartingClick( wxCommandEvent& event )
{
	JakubSchwarzmeier::Std::Graphics::RotationMatrix mat;
	JakubSchwarzmeier::Std::Math::Vector<> vec;

	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	mat = *(GetActiveView()->GetObserver()->GetCamera());

	vec = mat.GetPosition();
	m_startMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	doc->GetVisualization()->GetAnimation()->SetStartCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );

	m_startSnapshot = ::wxGetApp().GetMainFrame()->GetSnapshot();
	doc->GetVisualization()->GetAnimation()->SetStartSnapshot( m_startSnapshot );

	TransferDataToWindow();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_APPLYSTARTING
 */

void CCameraDlg::OnApplystartingClick( wxCommandEvent& event )
{
	CDigalexDoc * doc = ::wxGetApp().GetDocument();

	CGalaxyObserver * obs = doc->GetVisualization()->GetObserver();
	obs->SetCamera( *( doc->GetVisualization()->GetAnimation()->GetStartCamera() ) );
	GetActiveView()->SetObserver( obs );

	::wxGetApp().GetMainFrame()->SetSnapshot( doc->GetVisualization()->GetAnimation()->GetStartSnapshot() );

	GetActiveView()->ReRender();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GETENDING
 */

void CCameraDlg::OnGetendingClick( wxCommandEvent& event )
{
	JakubSchwarzmeier::Std::Graphics::RotationMatrix mat;
	JakubSchwarzmeier::Std::Math::Vector<> vec;

	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	mat = *(GetActiveView()->GetObserver()->GetCamera());

	vec = mat.GetPosition();
	m_endMatrix.sprintf(s_strSprintf.c_str(),  vec[0], vec[1], vec[2],
		mat.GetQuaternion().GetAngle(), mat.GetQuaternion().GetLongitude(), 
		mat.GetQuaternion().GetLatitude() );

	m_localChunkDesc->SetEndCamera( reinterpret_cast<CCamera*>( GetActiveView()->GetObserver()->GetCamera() ) );

	m_endSnapshot = ::wxGetApp().GetMainFrame()->GetSnapshot();
	m_localChunkDesc->SetEndSnapshot( m_startSnapshot );

	TransferDataToWindow();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_APPLYENDING
 */

void CCameraDlg::OnApplyendingClick( wxCommandEvent& event )
{
	CDigalexDoc * doc = ::wxGetApp().GetDocument();

	CGalaxyObserver * obs = doc->GetVisualization()->GetObserver();
	obs->SetCamera( *( m_localChunkDesc->GetEndCamera() ) );
	GetActiveView()->SetObserver( obs );

	::wxGetApp().GetMainFrame()->SetSnapshot( m_localChunkDesc->GetEndSnapshot() );

	GetActiveView()->ReRender();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CCameraDlg::OnOkClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CDigalexDoc * doc = ::wxGetApp().GetDocument();

		m_localChunkDesc->SetDuration( m_animationDuration ); 
		doc->GetVisualization()->GetAnimation()->SetStartSnapshot( m_startSnapshot );
		m_localChunkDesc->SetEndSnapshot( m_endSnapshot );

		GetParent()->Enable( true );
	
		Destroy();

		EndDialog( wxID_OK );
	} 	
}


/*!
 * Should we show tooltips?
 */

bool CCameraDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCameraDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCameraDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCameraDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCameraDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCameraDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCameraDlg icon retrieval
}

JakubSchwarzmeier::wx::COpenGLCanvas * CCameraDlg::GetActiveView()
{
	CMainFrame * frame = ::wxGetApp().GetMainFrame();
	wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
	CDigalexView * view = wx_reinterpret_cast( CDigalexView*, childFrame->GetView() );

	return view->GetCanvas();
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_GETSTARTING
 */

void CCameraDlg::OnUpdateUI( wxUpdateUIEvent& event )
{
	std::vector<CChunkDesc*> * chVec = ::wxGetApp().GetDocument()->GetVisualization()->GetAnimation()->GetChunks();
	CChunkDesc * ch = reinterpret_cast<CChunkDesc*>( GetClientData() );

	if (  (chVec->size() == 1) || ( chVec->front() == ch ) )
	{
		event.Enable( true );
	}
	else
	{
		event.Enable( false );
	}
}


std::string CCameraDlg::s_strSprintf( _T("%f %f %f %f %f %f") );
