/////////////////////////////////////////////////////////////////////////////
// Name:        colorsdlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     16/02/2007 11:55:59
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "colorsdlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "colorsdlg.h"

#include "DigalexApp.h"
#include "stararchive.h"

////@begin XPM images
////@end XPM images

/*!
 * CColorsDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CColorsDlg, wxDialog )

/*!
 * CColorsDlg event table definition
 */

BEGIN_EVENT_TABLE( CColorsDlg, wxDialog )

////@begin CColorsDlg event table entries
    EVT_BUTTON( XRCID("ID_ASSIGN"), CColorsDlg::OnAssignClick )
    EVT_UPDATE_UI( XRCID("ID_ASSIGN"), CColorsDlg::OnAssignUpdateUI )

    EVT_BUTTON( wxID_OK, CColorsDlg::OnOKClick )

////@end CColorsDlg event table entries

END_EVENT_TABLE()

/*!
 * CColorsDlg constructors
 */

CColorsDlg::CColorsDlg( )
{
    Init();
}

CColorsDlg::CColorsDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CColorsDlg creator
 */

bool CColorsDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CColorsDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CColorsDlg creation
    return true;
}

/*!
 * Member initialisation 
 */

void CColorsDlg::Init()
{
////@begin CColorsDlg member initialisation
    m_massCtrl = NULL;
    m_presetCtrl = NULL;
////@end CColorsDlg member initialisation
}
/*!
 * Control creation for CColorsDlg
 */

void CColorsDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/colorsdlg.xrc"));

////@begin CColorsDlg content construction
    if (!wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("ID_CCOLORSDLG")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    m_massCtrl = XRCCTRL(*this, "ID_MASS", wxListBox);
    m_presetCtrl = XRCCTRL(*this, "ID_PRESET", wxListBox);
////@end CColorsDlg content construction

	CStarArchive * pStarArchive = ::wxGetApp().GetDocument()->GetStarArchive();
	CStarArchive::SET_MASSES * pSetMasses = pStarArchive->GetMasses();

	CStarArchive::SET_MASSES::const_iterator it;
	for ( it = pSetMasses->begin( ) ; it != pSetMasses->end( ) ; it++ )
	{
		wxString mass;
		mass.sprintf("%.18g", (*it) );
		m_massCtrl->Append( mass );
	}

    // Create custom windows not generated automatically here.
////@begin CColorsDlg content initialisation
////@end CColorsDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CColorsDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CColorsDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CColorsDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CColorsDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CColorsDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CColorsDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CColorsDlg icon retrieval
}
/*!
 * wxEVT_UPDATE_UI event handler for ID_ASSIGN
 */

void CColorsDlg::OnAssignUpdateUI( wxUpdateUIEvent& event )
{
	bool bMass = m_massCtrl->GetSelection() != wxNOT_FOUND;
	bool bPreset = m_presetCtrl->GetSelection() != wxNOT_FOUND;
	event.Enable( bMass && bPreset );
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_ASSIGN
 */

void CColorsDlg::OnAssignClick( wxCommandEvent& event )
{
	CStarArchive * pStarArchive = ::wxGetApp().GetDocument()->GetStarArchive();

	wxString strMass = m_massCtrl->GetStringSelection();
	double mass = atof( strMass.c_str() );
	int iColorSeries = m_presetCtrl->GetSelection();

	CStarArchive::MAP_MASSES * pMapMasses = pStarArchive->GetMassesMap();
	CStarArchive::MAP_MASSES::const_iterator it = pMapMasses->find( mass );
	if ( it != pMapMasses->end() )
	{
		pMapMasses->erase( mass );
	}
	pMapMasses->insert( CStarArchive::MAP_MASSES::value_type( mass, iColorSeries ) );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CColorsDlg::OnOKClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		::wxGetApp().GetDocument()->GetStarArchive()->GetSnapShot( ::wxGetApp().GetMainFrame()->GetSnapshot() );

		EndDialog( wxID_OK );
	}
}


