/////////////////////////////////////////////////////////////////////////////
// Name:        mainframe.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     02/11/06 09:03:10
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "mainframe.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "mainframe.h"
#include "digalexapp.h"
#include "digalexview.h"
#include "app_resources.h"
#include "videomakerdlg.h"
#include "colorsdlg.h"
#include "videotextdescriptiondlg.h"

////@begin XPM images
////@end XPM images

/*!
 * CMainFrame type definition
 */

IMPLEMENT_CLASS( CMainFrame, wxDocMDIParentFrame )

/*!
 * CMainFrame event table definition
 */

BEGIN_EVENT_TABLE( CMainFrame, wxDocMDIParentFrame )

////@begin CMainFrame event table entries
    EVT_CLOSE( CMainFrame::OnFileExit )
    EVT_SIZE( CMainFrame::OnSize )

    EVT_SLIDER( XRCID("ID_SLIDER"), CMainFrame::OnSliderUpdated )

////@end CMainFrame event table entries

    EVT_MENU( XRCID("ID_OPEN"), CMainFrame::OnFileOpen)
    EVT_MENU( XRCID("ID_CLOSE"), CMainFrame::OnFileClose)
	EVT_MENU( XRCID("ID_EXIT"), CMainFrame::OnFileExit)

    EVT_MENU( XRCID("ID_CREATEMOVIE"), CMainFrame::OnViewMovie)
    EVT_MENU( XRCID("ID_COLORS"), CMainFrame::OnViewColors)
    EVT_MENU( XRCID("ID_VIDEOTEXT"), CMainFrame::OnViewVideoText)

END_EVENT_TABLE()

/*!
 * CMainFrame constructors
 */

CMainFrame::CMainFrame( )
{
}

CMainFrame::CMainFrame( wxDocManager *manager, wxFrame *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Create( manager, parent, id, caption, pos, size, style );
}

/*!
 * CMainFrame creator
 */

CMainFrame::~CMainFrame()
{
}

bool CMainFrame::Create( wxDocManager *manager, wxFrame *parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CMainFrame member initialisation
    m_slider = NULL;
////@end CMainFrame member initialisation
	m_camerasPanel = NULL;
	m_prevPos = -1;

////@begin CMainFrame creation
    SetParent(parent);
    CreateControls();
    Centre();
////@end CMainFrame creation
    return true;
}

/*!
 * Control creation for CMainFrame
 */

void CMainFrame::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/mainframe.xrc"));
    wxXmlResource::Get()->Load(wxT("xrc/menus.xrc"));

////@begin CMainFrame content construction
    if (!wxXmlResource::Get()->LoadObject(this, GetParent(), wxT("ID_MAINFRAME"), wxT("wxDocMDIParentFrame")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    m_slider = XRCCTRL(*this, "ID_SLIDER", wxSlider);
////@end CMainFrame content construction

    //  Set to this wxDocMDIParentFrame member m_docManager
    m_docManager = ::wxGetApp().GetDocManager(); 

    // associate the menu bar with the frame
	SetMenuBar( AppResources::CreatePartialMenubar() );

	m_camerasPanel = NULL;

	// A history of files visited. Use this menu.
	m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
	m_docManager->FileHistoryAddFilesToMenu();

	int x = wxSystemSettings::GetMetric( wxSYS_SCREEN_X );
	int y = wxSystemSettings::GetMetric( wxSYS_SCREEN_Y );

	SetSize( -1, -1, x * 0.95, y * 0.95 );

    // Create custom windows not generated automatically here.

////@begin CMainFrame content initialisation
////@end CMainFrame content initialisation
}


/*!
 * wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER
 */

void CMainFrame::OnSliderUpdated( wxCommandEvent& event )
{
	int pos = m_slider->GetValue();

	if (pos != m_prevPos)
	{

		CStarArchive * pStarArchive = ::wxGetApp().GetDocument()->GetStarArchive();
		pStarArchive->LockReading();
		pStarArchive->GetSnapShot( pos );
		pStarArchive->ReleaseReading();

		CMainFrame * frame = ::wxGetApp().GetMainFrame();
		wxDocMDIChildFrame * childFrame = wx_reinterpret_cast( wxDocMDIChildFrame*, frame->GetActiveChild() );
		CDigalexView * view = wx_reinterpret_cast( CDigalexView*, childFrame->GetView() );

		view->GetCanvas()->SetFocus();

		m_prevPos = pos;
	}
////@begin wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER in CMainFrame.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER in CMainFrame. 
}

int CMainFrame::GetSnapshot()
{
	return m_prevPos;
}

void CMainFrame::SetSnapshot(int no)
{
	m_slider->SetValue( no );
	OnSliderUpdated( wxCommandEvent() );
}

/*!
 * Should we show tooltips?
 */

bool CMainFrame::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CMainFrame::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CMainFrame bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CMainFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CMainFrame::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CMainFrame icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CMainFrame icon retrieval
}


CChildFrame * CMainFrame::CreateChildFrame(wxDocument * pDocument, int pViewClass)
{
	//  Get template class, which is a factory for creating new views
	wxDocTemplate * pTemplate;
	if (pViewClass < 0)
	{
		//  Newly created view will be of the same type as currently 
		//  active window.
		//  tady musi bejt zjistovacka jaky vokno je aktiv
		//  wxMDIChildFrame * pMDIActive = GetActiveChild();	// Get the currently active MDI child window. 
		pTemplate = pDocument->GetDocumentTemplate();
	}
	else
	{
		//  Newly created view will be other type than is the curretly opened window.
		wxList list = ::wxGetApp().GetDocManager()->GetTemplates();
		pTemplate = reinterpret_cast<wxDocTemplate*>( list.Item(pViewClass)->GetData() );
	}
    
	wxView * pView = pTemplate->CreateView( pDocument );
	CChildFrame * subframe = new CChildFrame(pDocument, pView, this);
	//  specialni ikona
	//    subframe->SetIcon(wxIcon(_T("chrt_icn")));
	//  specialni menu
    subframe->Show(true);
//	subframe->Update();		//  first redraw of frame and its child

	//  nekam zasejvovat subframe

	return subframe;
}
/*
wxMDIClientWindow * CMainFrame::OnCreateClient()
{
	return new CMDIClientWindow;
}

*/

void CMainFrame::SetSliderRange( int maxValue )
{
	m_slider->SetRange(0, maxValue);
}




/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_OPEN
 */

void CMainFrame::OnFileOpen( wxCommandEvent& event )
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnFileClose( event );
	}

	::wxGetApp().GetDocManager()->OnFileOpen( event );

	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnNewDocument();
	}

	m_camerasPanel->UpdateCameras();
}

void CMainFrame::OnFileClose( wxCommandEvent& event )
{
	::wxGetApp().GetDocManager()->OnFileClose( event );

	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 0 )
	{
		wxMenuBar * oldMenu = GetMenuBar();
		SetMenuBar( AppResources::CreatePartialMenubar() );

		m_docManager->FileHistoryRemoveMenu( oldMenu->GetMenu(0) );
		m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
		m_docManager->FileHistoryAddFilesToMenu();
		oldMenu->Destroy();

		GetSizer()->GetItem((size_t)0)->GetWindow()->Destroy();
		SetSizer( NULL );
		int w, h;
		GetClientSize(&w, &h);
		GetClientWindow()->SetSize(0, 0, w, h);

		m_camerasPanel = NULL;
	}
}

void CMainFrame::OnNewDocument()
{
	wxBoxSizer * mainSizer = new wxBoxSizer( wxHORIZONTAL );
	SetSizer( mainSizer );

	m_camerasPanel = new CCamerasPanel( this );
	mainSizer->Insert(0, m_camerasPanel, 0, wxGROW );
	Layout();

	//  initial size of mainframe
	int wPanel, hPanel;
	mainSizer->GetItem((size_t)0)->GetWindow()->GetSize( &wPanel, &hPanel );

	int w, h;
	GetClientSize(&w, &h);
	GetClientWindow()->SetSize(wPanel, 0, w - wPanel, h);

	//  create full menu
	wxMenuBar * oldMenu = GetMenuBar();
	SetMenuBar( AppResources::CreateFullMenubar() );

	m_docManager->FileHistoryRemoveMenu( oldMenu->GetMenu(0) );
    m_docManager->FileHistoryUseMenu( GetMenuBar()->GetMenu(0) );
	m_docManager->FileHistoryAddFilesToMenu();
	oldMenu->Destroy();
}
/*!
 * wxEVT_SIZE event handler for ID_MAINFRAME
 */

void CMainFrame::OnSize( wxSizeEvent& event )
{
	if ( GetSizer() != NULL )
	{
		int wPanel, hPanel;
		GetSizer()->GetItem((size_t)0)->GetWindow()->GetSize( &wPanel, &hPanel );

		int w, h;
		GetClientSize(&w, &h);
		GetClientWindow()->SetSize(wPanel, 0, w - wPanel, h);
		Layout();

		#ifdef __WXUNIVERSAL__   
			event.Skip();
		#endif	
	}
	else
	{
		event.Skip();
	}
}


void CMainFrame::OnFileExit( wxCloseEvent& event )
{
	if ( ::wxGetApp().GetDocManager()->GetDocuments().GetCount() == 1 )
	{
		OnFileClose( wxCommandEvent() );
	}

	event.Skip();
}

void CMainFrame::OnFileExit(wxCommandEvent& event)
{
	OnFileExit( wxCloseEvent() );

	wxDocMDIParentFrame::OnExit(event);
}

void CMainFrame::OnViewMovie(wxCommandEvent& event)
{
	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();

	CVideoMakerDlg dlg;
	dlg.SetClientData( vis );
	dlg.Create( this );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->Modify( TRUE );
	}
}

void CMainFrame::OnViewColors(wxCommandEvent& event)
{
	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();

	CColorsDlg dlg;
	dlg.Create( this );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->Modify( TRUE );
	}
}

void CMainFrame::OnViewVideoText(wxCommandEvent& event)
{
	CDigalexDoc * doc = ::wxGetApp().GetDocument();
	CVisualization * vis = doc->GetVisualization();

	CVideoTextDescriptionDlg dlg;
	dlg.Create( this );
	if ( dlg.ShowModal() == wxID_OK )
	{
		doc->Modify( TRUE );
	}
}

