/////////////////////////////////////////////////////////////////////////////
// Name:        videotextdescriptiondlg.cpp
// Purpose:     
// Author:      Jakub Schwarzmeier
// Modified by: 
// Created:     20/02/2007 15:31:31
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma implementation "videotextdescriptiondlg.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "videotextdescriptiondlg.h"
#include "digalexapp.h"
#include "threedcanvas.h"

////@begin XPM images
////@end XPM images

/*!
 * CVideoTextDescriptionDlg type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CVideoTextDescriptionDlg, wxDialog )

/*!
 * CVideoTextDescriptionDlg event table definition
 */

BEGIN_EVENT_TABLE( CVideoTextDescriptionDlg, wxDialog )

////@begin CVideoTextDescriptionDlg event table entries
    EVT_UPDATE_UI( XRCID("ID_TITLE"), CVideoTextDescriptionDlg::OnUpdateUI )

    EVT_UPDATE_UI( XRCID("ID_AFFILATION"), CVideoTextDescriptionDlg::OnUpdateUI )

    EVT_UPDATE_UI( XRCID("ID_AUTHOR"), CVideoTextDescriptionDlg::OnUpdateUI )

    EVT_UPDATE_UI( XRCID("ID_TIMELINE"), CVideoTextDescriptionDlg::OUpdateTimeLine )

    EVT_BUTTON( wxID_OK, CVideoTextDescriptionDlg::OnOKClick )

////@end CVideoTextDescriptionDlg event table entries

END_EVENT_TABLE()

/*!
 * CVideoTextDescriptionDlg constructors
 */

CVideoTextDescriptionDlg::CVideoTextDescriptionDlg( )
{
    Init();
}

CVideoTextDescriptionDlg::CVideoTextDescriptionDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

/*!
 * CVideoTextDescriptionDlg creator
 */

bool CVideoTextDescriptionDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVideoTextDescriptionDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    SetParent(parent);
    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CVideoTextDescriptionDlg creation
    return true;
}

/*!
 * Member initialisation 
 */

void CVideoTextDescriptionDlg::Init()
{
////@begin CVideoTextDescriptionDlg member initialisation
    m_timeCoeff = 1.0;
    m_ctrlUseVideoDesc = NULL;
    m_ctrlTime = NULL;
    m_ctrlUseGrid = NULL;
    m_ctrlCameraParameters = NULL;
////@end CVideoTextDescriptionDlg member initialisation
}
/*!
 * Control creation for CVideoTextDescriptionDlg
 */

void CVideoTextDescriptionDlg::CreateControls()
{    
    wxXmlResource::Get()->Load(wxT("xrc/videotextdlg.xrc"));


////@begin CVideoTextDescriptionDlg content construction
    if (!wxXmlResource::Get()->LoadDialog(this, GetParent(), _T("ID_CVIDEOTEXTDESCRIPTIONDLG")))
        wxLogError(wxT("Missing wxXmlResource::Get()->Load() in OnInit()?"));
    m_ctrlUseVideoDesc = XRCCTRL(*this, "ID_USEVIDEOTEXT", wxCheckBox);
    m_ctrlTime = XRCCTRL(*this, "ID_TIME", wxCheckBox);
    m_ctrlUseGrid = XRCCTRL(*this, "ID_GRID", wxCheckBox);
    m_ctrlCameraParameters = XRCCTRL(*this, "ID_CAMERAPARAMETERS", wxCheckBox);
    // Set validators
    if (FindWindow(XRCID("ID_TITLE")))
        FindWindow(XRCID("ID_TITLE"))->SetValidator( wxTextValidator(wxFILTER_NONE, & m_strTitle) );
    if (FindWindow(XRCID("ID_AFFILATION")))
        FindWindow(XRCID("ID_AFFILATION"))->SetValidator( wxTextValidator(wxFILTER_NONE, & m_strAffilation) );
    if (FindWindow(XRCID("ID_AUTHOR")))
        FindWindow(XRCID("ID_AUTHOR"))->SetValidator( wxTextValidator(wxFILTER_NONE, & m_strAuthor) );
    if (FindWindow(XRCID("ID_TIMELINE")))
        FindWindow(XRCID("ID_TIMELINE"))->SetValidator( wxDoubleValidator( & m_timeCoeff ) );
////@end CVideoTextDescriptionDlg content construction

	// We're using wxConfig's "create-on-demand" feature: it will create the
	// config object when it's used for the first time. It has a number of
	// advantages compared with explicitly creating our wxConfig:
	//  1) we don't pay for it if we don't use it
	//  2) there is no danger to create it twice
	wxConfigBase * pConfig = wxConfigBase::Get();

	pConfig->Read( CVideoTextDescriptionDlg::m_sTitle, &m_strTitle );
	pConfig->Read( CVideoTextDescriptionDlg::m_sAffilation, &m_strAffilation );
	pConfig->Read( CVideoTextDescriptionDlg::m_sAuthor, &m_strAuthor );

    // Create custom windows not generated automatically here.
////@begin CVideoTextDescriptionDlg content initialisation
////@end CVideoTextDescriptionDlg content initialisation
}

/*!
 * Should we show tooltips?
 */

bool CVideoTextDescriptionDlg::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVideoTextDescriptionDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVideoTextDescriptionDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVideoTextDescriptionDlg bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVideoTextDescriptionDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVideoTextDescriptionDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVideoTextDescriptionDlg icon retrieval
}
/*!
 * wxEVT_UPDATE_UI event handler for ID_TITLE
 */

void CVideoTextDescriptionDlg::OnUpdateUI( wxUpdateUIEvent& event )
{
	event.Enable( m_ctrlUseVideoDesc->IsEnabled() );
}
/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CVideoTextDescriptionDlg::OnOKClick( wxCommandEvent& event )
{
	if ( Validate() && TransferDataFromWindow() )
	{
		CThreedCanvas * tc = ::wxGetApp().GetDocument()->GetVisualization()->GetThreedCanvas();

		if (m_ctrlUseVideoDesc->GetValue())
		{
			//  save to registry
			wxConfigBase * pConfig = wxConfigBase::Get();
			pConfig->Read( CVideoTextDescriptionDlg::m_sTitle, &m_strTitle );
			pConfig->Read( CVideoTextDescriptionDlg::m_sAffilation, &m_strAffilation );
			pConfig->Read( CVideoTextDescriptionDlg::m_sAuthor, &m_strAuthor );

			//  save to video rendering engine
			tc->SetTitle( m_strTitle.c_str() );
			tc->SetAffilation( m_strAffilation.c_str() );
			tc->SetAuthor( m_strAuthor.c_str() );

			tc->SetUseVideoDesc( true );
		}
		else
		{
			tc->SetUseVideoDesc( false );
		}

		if ( m_ctrlTime->GetValue() )
		{
			tc->SetUseTime( true );
			tc->SetTimeCoefficient( m_timeCoeff );
		}
		else
		{
			tc->SetUseTime( false );
		}

		if ( m_ctrlUseGrid->GetValue() )
		{
			tc->SetUseGrid( true );
		}
		else
		{
			tc->SetUseGrid( false );
		}

		if ( m_ctrlCameraParameters->GetValue() )
		{
			tc->SetUseCameraParameters( true );
		}
		else
		{
			tc->SetUseCameraParameters( false );
		}

		EndDialog( wxID_OK );
	}
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_TIMELINE
 */

void CVideoTextDescriptionDlg::OUpdateTimeLine( wxUpdateUIEvent& event )
{
	event.Enable(  m_ctrlTime->IsEnabled() );
}




const wxString CVideoTextDescriptionDlg::m_sTitle( wxT("/Settings/VideoTextDescription/Title") );
const wxString CVideoTextDescriptionDlg::m_sAffilation( wxT("/Settings/VideoTextDescription/Affilation") );
const wxString CVideoTextDescriptionDlg::m_sAuthor( wxT("/Settings/VideoTextDescription/Author") );
