#include "../../inc/schwarzmeier/excel.h"

std::string C1R1ToExcel(int column, int row)
{
	row += 1;
	//  column is modified by the design of this function, no need for +1 here

	int first = 0;
	while (column > 25 )
	{
		first++;
		column -= 26;
	}

	TCHAR sz[16];
	sprintf(sz, "%c", 64 + first );
	std::string strFirst( sz );

	sprintf(sz, "%c", 65 + column  );
	std::string strSecond( sz );

	sprintf(sz, "%d", row );
	strSecond += sz;

	if (first > 0)
	{
		strFirst += strSecond;
		return strFirst;
	}
	else
		return strSecond;
}

Excel::_ApplicationPtr CreateExcel()
{
	Excel::_ApplicationPtr pExcel;
	pExcel.CreateInstance(L"Excel.Application");

	return pExcel;
}

void QuitExcel(Excel::_ApplicationPtr pExcel)
{
	if (pExcel != NULL)
	{
		//  Make the application visible and give the user control of Microsoft Excel.
		pExcel->PutVisible( 0, VARIANT_TRUE );
		pExcel->UserControl = VARIANT_TRUE;

		pExcel->ActiveWorkbook->Saved[0] = VARIANT_TRUE;
		pExcel->Quit();
	}
}

Excel::_WorkbookPtr AddWorkbook(Excel::_ApplicationPtr pExcel)
{
	Excel::WorkbooksPtr pWorkBooks = pExcel->Workbooks;
	Excel::_WorkbookPtr pWorkbook  = pWorkBooks->Add((long)Excel::xlWorksheet);
	return pWorkbook;
}

void SetCellValue(Excel::_WorksheetPtr pWorksheet, int col, int row, std::string value)
{
	pWorksheet->Cells->_Default[row+1][col+1] = value.c_str();
}

void SetCellValue(Excel::_WorksheetPtr pWorksheet, int col, int row, double value)
{
	pWorksheet->Cells->_Default[row+1][col+1] = value;
}

std::string GetCellValue(Excel::_WorksheetPtr pWorksheet, size_t col, size_t row)
{
	std::string ret = (char*)bstr_t( pWorksheet->Cells->_Default[row+1][col+1] );
	return ret;
}

Excel::_ChartPtr CreateChart( Excel::_WorkbookPtr pWorkbook, Excel::_WorksheetPtr pWorksheet,
		std::string chartTitle, std::string xlabel, std::string ylabel )
{
	using namespace Excel;

	RangePtr pRange = pWorksheet->GetRange("A1:A2");	//  dummy initial range
	std::string save[4];
	save[0] = GetCellValue(pWorksheet, 0, 0);
	save[1] = GetCellValue(pWorksheet, 0, 1);
	save[2] = GetCellValue(pWorksheet, 1, 0);
	save[3] = GetCellValue(pWorksheet, 1, 1);
	
	SetCellValue(pWorksheet, 0, 0, 1.0);
	SetCellValue(pWorksheet, 0, 1, 2.0);
	SetCellValue(pWorksheet, 1, 0, 3.0);
	SetCellValue(pWorksheet, 1, 1, 8.0);

	//  add chart
	_ChartPtr pChart = pWorkbook->Charts->Add();

	pChart->ChartWizard((Range*) pRange, (long) xlXYScatter, 2L, (long) xlColumns,
		0, 0, 1L, chartTitle.c_str(), xlabel.c_str(), ylabel.c_str());

	SetCellValue(pWorksheet, 0, 0, save[0]);
	SetCellValue(pWorksheet, 0, 1, save[1]);
	SetCellValue(pWorksheet, 1, 0, save[2]);
	SetCellValue(pWorksheet, 1, 1, save[3]);

	return pChart;
}

void SetSeriesValues( Excel::SeriesPtr series, Excel::_WorksheetPtr pSheet, 
					 size_t xColStart, size_t xRowStart, size_t xColEnd, size_t xRowEnd, 
					 size_t yColStart, size_t yRowStart, size_t yColEnd, size_t yRowEnd )
{
	series->XValues = (Excel::Range*)pSheet->Range[C1R1ToExcel(xColStart, xRowStart).c_str()][C1R1ToExcel(xColEnd, xRowEnd).c_str()];
	series->Values = (Excel::Range*)pSheet->Range[C1R1ToExcel(yColStart, yRowStart).c_str()][C1R1ToExcel(yColEnd, yRowEnd).c_str()];
}

Excel::RangePtr GetRange( Excel::_WorksheetPtr pSheet, size_t columnStart, size_t rowStart, size_t columnEnd, size_t rowEnd )
{
	return pSheet->Range[C1R1ToExcel(columnStart, rowStart).c_str()][C1R1ToExcel(columnEnd, rowEnd).c_str()];
}

void excel_dump_com_error(_com_error &e)
{
    _tprintf(_T("Oops - hit an error!\n"));
    _tprintf(_T("\a\tCode = %08lx\n"), e.Error());
    _tprintf(_T("\a\tCode meaning = %s\n"), e.ErrorMessage());
    _bstr_t bstrSource(e.Source());
    _bstr_t bstrDescription(e.Description());
    _tprintf(_T("\a\tSource = %s\n"), (LPCTSTR) bstrSource);
    _tprintf(_T("\a\tDescription = %s\n"), (LPCTSTR) bstrDescription);
}

struct StartOle
{
	StartOle() { CoInitialize(NULL); }
	~StartOle() { CoUninitialize(); }
} _inst_StartOle; 
