#include "../../inc/schwarzmeier/xmllib.h"

#include <string>
#include <iomanip>

//  From tchar.h
#ifndef _T
	#define _T
#endif

#define START_LOC	\

#define FINISH_LOC \

std::string GetXPathResponse( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	std::string result;
	if(!xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		xmlChar * textValue = xmlNodeListGetString(pXmlDoc, xpath->nodesetval->nodeTab[0], 1);
		result = ((const char *)textValue);
		xmlFree(textValue);
	}
	else
	{
		result = _T("");
	}


	return result;
}

xmlNode * SelectSingleNode( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		return NULL;
	}

	return xpath->nodesetval->nodeTab[0];
}

xmlNodeSet * SelectNodes( xmlDoc * pXmlDoc, xmlNode * pNode, std::string strXPath )
{
	xmlXPathContext * context = xmlXPathNewContext(pXmlDoc);
	context->node = pNode;
	xmlXPathObject * xpath = xmlXPathEvalExpression( (const xmlChar *)strXPath.c_str(), context);
	xmlXPathFreeContext(context);

	if(xmlXPathNodeSetIsEmpty(xpath->nodesetval))
	{
		return NULL;
	}

	return xpath->nodesetval;
}

void SetAttribute( xmlNode * pNode, std::string attName, double attValue )
{
	START_LOC

	char szTmp[32];
	sprintf(szTmp, "%.18e", attValue );
	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );

	FINISH_LOC
}

void SetAttribute( xmlNode * pNode, std::string attName, int attValue )
{
	char szTmp[32];
	sprintf(szTmp, "%d", attValue );
	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );
}

void SetAttribute( xmlNode * pNode, std::string attName, long long attValue )
{
	char szTmp[32];
	#ifndef _MSC_VER
		sprintf(szTmp, "%lld", attValue );
	#else
		sprintf(szTmp, "%I64d", attValue );
	#endif

	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );
}


void SetAttribute( xmlNode * pNode, std::string attName, bool attValue )
{
	char szTmp[32];
	if (attValue)
	{
		strcpy( szTmp, "true" );
	}
	else
	{
		strcpy( szTmp, "false" );
	}

	xmlNewProp( pNode, (const xmlChar *)attName.c_str(), (const xmlChar *)szTmp );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, std::string textValue )
{
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)textValue.c_str() );
	xmlAddChild( pNode, textNode );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, int textValue )
{
	char szTmp[32];
	sprintf(szTmp, "%d", textValue );
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)szTmp );
	xmlAddChild( pNode, textNode );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, long long textValue )
{
	char szTmp[32];

	#ifndef _MSC_VER
		sprintf(szTmp, "%lld", textValue );
	#else
		sprintf(szTmp, "%I64d", textValue );
	#endif
	
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)szTmp );
	xmlAddChild( pNode, textNode );
}

void SetTextNode( xmlDoc * pXmlDoc, xmlNode * pNode, double textValue )
{
	START_LOC

	char szTmp[32];
	sprintf(szTmp, "%.18e", textValue );
	xmlNode * textNode = xmlNewDocText( pXmlDoc, (const xmlChar *)szTmp );
	xmlAddChild( pNode, textNode );

	FINISH_LOC
}

void GetAttribute( xmlNode * pNode, std::string attName, double * retValue )
{
	START_LOC

	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	*retValue = atof( (const char*) attr );
	xmlFree(attr);

	FINISH_LOC
}

void GetAttribute( xmlNode * pNode, std::string attName, int * retValue )
{
	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	*retValue = atoi( (const char*) attr );
	xmlFree(attr);
}

void GetAttribute( xmlNode * pNode, std::string attName, long long * retValue )
{
	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	*retValue = static_cast<long long>( atof( (const char*) attr ) );
	xmlFree(attr);
}

void GetAttribute( xmlNode * pNode, std::string attName, bool * retValue )
{
	xmlChar * attr = xmlGetProp(pNode, (const xmlChar *) attName.c_str() );
	if ( ( strcmp( reinterpret_cast<const char *>(attr), "true" ) == 0 ) || ( strcmp( reinterpret_cast<const char *>(attr), "1" ) == 0 ) )
	{
		*retValue = true;
	}
	else
	{
		*retValue = false;
	}

	xmlFree(attr);
}

