/*************************************************************************************
 *
 *  initialconditions.h:  base class for initial conditions
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#ifndef _ICSLIB_INITIALCONDITIONS_H
#define _ICSLIB_INITIALCONDITIONS_H

#include "../../src/lib/includes.h"
#include "../../src/lib/plotout.h"

#define TIMES_RADII 10

class CInitialConditions
{
public:
	CInitialConditions(void);
	virtual ~CInitialConditions(void);

	virtual void Generate();	//  create model

	//  tabulation of values
	void SetFuncTabs( unsigned int funcTabs );
	unsigned int GetFuncTabs( );
	void SetCount( unsigned int count );
	unsigned int GetCount( );

	void Set_r(double r);

	double Get_rMax();
	double GetTotalMass();

	void SetTotalMass(double massTotal);

	void Set_a(double a);
	double Get_a();

	//  output
	void SetOutputName( std::string outputName );
	std::string GetOutputName( );
	void SetPlot( bool bUsePlot );
	void SetPlotTabs( unsigned int plotTabs );

	static const std::string s_simNameExt;

protected:

	unsigned int m_count;
	unsigned int m_funcTabs;	//  smootheness of tabelated values
	unsigned int m_plotTabs;	
    
	double m_rMax;	//  maximum radius of galaxy 
	double m_rCut;
	double m_totalMass;	//  mass of galaxy
	double m_G;		//  gravitational constant
	double m_a;

	std::string m_name;	//  name of model

	std::string m_outputName;	//  name of output file
	bool m_bUsePlot;

	CPlotOut m_plotOut;

	std::string StartBinary();
	std::ofstream m_fOut;
	void StopBinary();
	void BinaryWrite(double val);

	xmlNode * StartXml();
	xmlDoc * m_pXmlDoc;
	void StopXml(xmlNode * pRootNode);
	void PrintFinish( double & Ekin, double & Epot );

	static const std::string s_simulation;
};

#endif //	_ICSLIB_INITIALCONDITIONS_H
