/*************************************************************************************
 *
 *  sphericalmodel.h:  suitable for deriving spherical models
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#ifndef _ICSLIB_SPHERICALMODEL_H
#define _ICSLIB_SPHERICALMODEL_H

#include "initialconditions.h"

class CSphericalModel: public CInitialConditions
{
public:

	CSphericalModel(void);
	virtual ~CSphericalModel(void);

	virtual void Generate();

	virtual double rho( double r ) = 0;

	void TabulateCumulativeMass();
	virtual double PrepCumulativeMass( double r );
	virtual double CumulativeMass( double r );
	
	void Tabulate_rm();
	double rm( double m );

	void TabulatePotential();
	double dphi_dr( double r );
	double PrePotential( double r );
	virtual double Potential( double r );

	void Tabulate_rho_phi();
	double d_rho_d_phi( double p );

    void TabulateDistributionFunction();
	double energy( double p );
	virtual double f( double p );

    virtual void RealizeModel();

	static double log(double x);	//  logarithmical grid spacing
	static double RandomGaussian(double mean, double stddev);

	//

	void SetPotential( gsl_spline * phir );

protected:

	double m_E;

	gsl_spline * m_mr;	//  m(r)
    gsl_interp_accel * m_accMr;
	gsl_spline * m_Rm;	//  r(m)
    gsl_interp_accel * m_accRm;

	gsl_spline * m_Phir;	//  phi(r)
    gsl_interp_accel * m_accPhir;

	gsl_spline * m_Rhophi;	
    gsl_interp_accel * m_accRhophi;
	gsl_spline * m_Fp;	//  f(p)
    gsl_interp_accel * m_accFp;

	gsl_integration_workspace * m_wspc; //  work space for integration
};

#endif //  _ICSLIB_SPHERICALMODEL_H
