/*************************************************************************************
 *
 *  encounter.cpp:  base class for encounter initial conditions
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "../includes.h"

#include "../../../inc/ics/encounter.h"
#include "../../../../amon/inc/simulation/bh.h"

CEncounter::CEncounter(void)
{
	m_name = "Encounter";
}

CEncounter::~CEncounter(void)
{
}

Math::Vector<> CEncounter::GetCenterOfMass(std::string fileName)
{
	//  Read parameters from XML file
	xmlDoc * pXmlDoc = xmlParseFile( (const char *)fileName.c_str() );
	xmlNode * pRootNode = xmlDocGetRootElement(pXmlDoc);

	//  get binary filename
	std::string strFilename = GetXPathResponse( pXmlDoc, pRootNode, "./filename/text()" );

	//  get last snapshot
	xmlNode * lastNode = SelectSingleNode( pXmlDoc, pRootNode, "./snapshot[last()]" );

	int startPos, stopPos;
	GetAttribute( lastNode,  "start", &startPos );
	GetAttribute( lastNode,  "end", &stopPos );

	//  input binary stream
	int seekPos = startPos;

	std::fstream binIn;
	binIn.open(strFilename.c_str(), std::ios_base::in | std::ios_base::binary );

	binIn.seekg( seekPos, std::ios_base::beg );

	JakubSchwarzmeier::Simulations::BarnesHut::DataReader dr;
	dr.OpenStream( &binIn );

	//  find center of mass of central gallaxy
	//
	Vector<> numerator = 0;
	double mass = 0;

	while (seekPos < stopPos)
	{
		JakubSchwarzmeier::Simulations::BarnesHut::Leaf leaf;
		leaf.Read(&dr);

		mass += leaf.m_mass;
		//  multiplication of vector with number
		numerator += leaf.m_mass * leaf.m_position;
	
		seekPos = binIn.tellg();
	}

	Math::Vector<> cm = numerator / mass;	//  "center of mass" (geometric quantity)
	
	return cm;
}
