/*************************************************************************************
 *
 *  plotout.cpp:  output values for plotting to gnuplut and Excel
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "includes.h"

#include "plotout.h"

CPlotOut::CPlotOut()
{
}

CPlotOut::~CPlotOut()
{
	#if (USE_EXCEL == 1)
		QuitExcel( m_pExcel );
	#endif
}

void CPlotOut::Create( std::string outputName )
{
	m_outputName = outputName;

	#if (USE_EXCEL == 1)
		try
		{
			m_pExcel = CreateExcel();
			m_pExcel->PutVisible( 0, VARIANT_TRUE );
			m_pWorkbook = AddWorkbook( m_pExcel );
			m_pWorksheet = m_pExcel->ActiveSheet;		
		}
		catch(_com_error &e)
		{
			excel_dump_com_error(e);
			getchar();
			QuitExcel( m_pExcel );
			exit(-1);
		}

		m_column = 0;
	#endif
}

void CPlotOut::StartPlot(std::string xlabel, std::string ylabel)
{
	m_xlabel = xlabel;
	m_ylabel = ylabel;
	std::string outFile;
	if ( m_outputName.empty() )  throw ;
	outFile = m_outputName + std::string("-") + ylabel + std::string("(") + xlabel + std::string(").txt");
	m_plotStream.open( outFile.c_str() );
	m_plotStream.setf(std::ios::scientific);

	#if (USE_EXCEL == 1)
		try 
		{
			//  create labels
			SetCellValue( m_pWorksheet, m_column, 0, m_xlabel );
			SetCellValue( m_pWorksheet, m_column+1, 0, m_ylabel );

			m_row = 2;	//  starting row for values, labels are one line higher
		} 
		catch(_com_error &e) 
		{
			excel_dump_com_error(e);
			getchar();
			QuitExcel( m_pExcel );
			exit(-1);
		}
	#endif
}

void CPlotOut::StopPlot()
{
	m_plotStream.close();

	#if (USE_EXCEL == 1)
		try
		{
			using namespace Excel;

			//  Create plot (chart)
			std::string strLabel = m_ylabel + std::string("(") + m_xlabel + std::string(")");
			std::string plotLabel = std::string("Plot ") + strLabel;

			m_pChart = CreateChart( m_pWorkbook, m_pWorksheet, plotLabel, m_xlabel, m_ylabel );
			m_pChart->PutName( strLabel.c_str() );

			Excel::SeriesCollectionPtr collection = m_pChart->SeriesCollection();
			Excel::SeriesPtr set = collection->Item(1);

			//  set x- and y-values
			SetSeriesValues( set, m_pWorksheet, 
				/* x-axis */
				m_column	/*start column*/,
				2			/*start row*/, 
				m_column	/*end column*/, 
				m_row-1		/*end row*/,

				/* y-axis */
				m_column+1	/*start column*/,
				2			/*start row*/, 
				m_column+1	/*end column*/, 
				m_row-1		/*end row*/
				);

			//  set marker style
			set->PutMarkerStyle( Excel::xlMarkerStyleNone );

			m_column+=3;
		}
		catch(_com_error &e) 
		{
			excel_dump_com_error(e);
			getchar();
			QuitExcel( m_pExcel );
			exit(-1);
		}
	#endif
}

void CPlotOut::Plot(double x, double y)
{
	m_plotStream << x << " " << y << std::endl;

	#if (USE_EXCEL == 1)
		try
		{
			//  fill value
			SetCellValue( m_pWorksheet, m_column, m_row, x );
			SetCellValue( m_pWorksheet, m_column+1, m_row++, y );
		} 
		catch(_com_error &e) 
		{
			excel_dump_com_error(e);
			getchar();
			QuitExcel( m_pExcel );
			exit(-1);
		}

	#endif
}
