/*************************************************************************************
 *
 *  compositemodel.cpp:  create galaxy from various components
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "../includes.h"

#include "../../inc/ics/units-convertor.h"

#include "../../../../amon/inc/simulation/bh.h"

CUnitsConvertor::CUnitsConvertor()
{
	m_name = "UnitsConvertor";

	m_G = 1.0;
	m_len = 1.0;
	m_mass = 1.0;
}

CUnitsConvertor::~CUnitsConvertor()
{
}

void CUnitsConvertor::SetInputName( std::string input )
{
	m_input = input;
}

void CUnitsConvertor::SetUnits( double G, double len, double mass )
{
	m_G = G;
	m_len = len;
	m_mass = mass;
}

void CUnitsConvertor::Generate()
{
	double G_C = m_G;
	double M_C = m_mass;
	double pos_C = m_len;
	double vel_C = sqrt( (m_G * m_mass)/m_len );

	xmlNode * pRootNode;
	long long seekPos;
	std::fstream binIn;
	JakubSchwarzmeier::Simulations::BarnesHut::DataReader dr;

	//  output binary stream
	std::string binICFile( m_outputName );
	binICFile += s_simNameExt;

	std::fstream binOut;
	binOut.open(binICFile.c_str(), std::ios_base::out | std::ios_base::binary );
	JakubSchwarzmeier::Simulations::BarnesHut::DataWriter dw;
	dw.OpenStream( &binOut );

	//  Read parameters from XML file
	m_pXmlDoc = xmlParseFile( (const char *)m_input.c_str() );
	pRootNode = xmlDocGetRootElement(m_pXmlDoc);

	//  get binary filename
	std::string strFilename = GetXPathResponse( m_pXmlDoc, pRootNode, "./filename/text()" );

	//  get last snapshot
	xmlNode * lastNode = SelectSingleNode( m_pXmlDoc, pRootNode, "./snapshot[last()]" );

	long long startPos, stopPos;
	GetAttribute( lastNode,  "start", &startPos );
	GetAttribute( lastNode,  "end", &stopPos );

	//  modify out filename
	std::string outFileNameBin(m_outputName);
	outFileNameBin += ".bin";
	xmlNode * fn = SelectSingleNode( m_pXmlDoc, pRootNode, "./filename" );

	xmlUnlinkNode( fn );
	xmlFreeNode( fn );

	xmlNodeSetContent(fn, (const xmlChar *) outFileNameBin.c_str() );

	seekPos = startPos;

	binIn.open(strFilename.c_str(), std::ios_base::in | std::ios_base::binary );
	binIn.seekg( seekPos, std::ios_base::beg );

	dr.OpenStream( &binIn );

	while (seekPos < stopPos)
	{
		JakubSchwarzmeier::Simulations::BarnesHut::Leaf leaf;
		leaf.Read(&dr);

		leaf.m_position = pos_C * leaf.m_position;
		leaf.m_velocity = vel_C * leaf.m_velocity;
		leaf.m_mass = M_C * leaf.m_mass;

		leaf.Write(&dw);

		m_count++;
		seekPos = binIn.tellg();
	}

	binIn.close();
	dr.close();

	xmlNodeSet * pNodes = SelectNodes( m_pXmlDoc, pRootNode, "./snapshot" );

	//  clearing itself
	for(int i = 0; i < pNodes->nodeNr; i++)
	{
		xmlUnlinkNode(pNodes->nodeTab[i]);
		xmlFreeNode(pNodes->nodeTab[i]);
	}

	StopXml(pRootNode);
}
