/*************************************************************************************
 *
 *  main-encounter.cpp:  defines the entry point for the Encounter application
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "../includes.h"

#include "../../inc/ics/directcollision.h"
#include "../../inc/ics/keplerencounter.h"

int main(int argc, char *argv[])
{
	//--  Choose a correct model.
	switch ( tolower( *argv[1] ) )
	{
	case 'd':
		{
			CDirectCollision dc;

			dc.SetCentral( argv[2] );
			dc.SetCompanion( argv[3] );
			dc.SetCollision( atof( argv[5] ), atof( argv[6] ), atof( argv[7] ) );

			dc.SetOutputName( argv[4] );

			dc.Generate( );
		}

		break;

	case 'k':
		{
			CKeplerEncounter ke;

			ke.SetCollision( atof( argv[5] ), atof( argv[6] ), atof( argv[7] ) );

			ke.SetCentral( argv[2] );
			ke.SetCompanion( argv[3] );

			ke.SetOutputName( argv[4] );

			ke.Generate( );
		}

		break;

	}	//  end switch

	return 0;
}
