/*************************************************************************************
 *
 *  main-flat.cpp:  defines the entry point for the flat models application
 *
 *  Copyright (c) 2007 Jakub Schwarzmeier
 *  University of West Bohemia, Department of General Physics
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#include "../includes.h"

#include "../../inc/ics/keplermodel.h"
#include "../../inc/ics/exponentialmodel.h"
#include "../../inc/ics/kuzminmodel.h"


void CmdParSetCount( CInitialConditions * model, const char * count )
{
	model->SetCount( atoi( count ) );
}

void CmdParSetOutputName( CInitialConditions * model, const char * outputName )
{
	model->SetOutputName( outputName );
}

void CmdParSetPlot( CInitialConditions * model, const char * bPlot )
{
	if ( tolower( *bPlot ) == 'p' )	model->SetPlot( true );
}

void help()
{
	std::cout << "Usage: icsmain model_code [model_parameter_1] [model_parameter_2] ... [model_parameter_N] output_file [p]" << std::endl;
	std::cout << "e.g. icsmain p 10000 ./bulge_component p" << std::endl;

	return;
}

int main(int argc, char *argv[])
{
    if (argc < 2)
	{
		//--  Probably a new user or wrong number of parameters. Print help message.
		help();
		return -1;
	}
	else
	{	
		//--  Choose a correct model.
		switch ( tolower( *argv[1] ) )
		{
		case 'k':
			if (argc > 3)
			{
				//--  Kepler's model
				CKeplerModel kepler;

				CmdParSetOutputName( &kepler, argv[2] );
				CmdParSetCount( &kepler, argv[3] );
				if (argc == 5)  CmdParSetPlot( &kepler, argv[4] );
				kepler.Generate();
			}
			else
			{
				help();
				return -1;
			}

			break;

		case 'u':
			//--  Kuzmin's model
			if (argc > 3)
			{
				CKuzminModel kuzmin;

				kuzmin.Activate(1.5);

				CmdParSetOutputName( &kuzmin, argv[2] );
				CmdParSetCount( &kuzmin, argv[3] );
				if (argc == 5)  CmdParSetPlot( &kuzmin, argv[4] );
				kuzmin.Generate();
			}
			else
			{
				help();
				return -1;
			}

			break;

		case 'e':
			//--  Exponential model
			if (argc > 3)
			{
				CExponentialModel exp;

				exp.Activate(1.5);

				CmdParSetOutputName( &exp, argv[2] );
				CmdParSetCount( &exp, argv[3] );
				if (argc == 5)  CmdParSetPlot( &exp, argv[4] );
				exp.Generate();
			}
			else
			{
				help();
				return -1;
			}

			break;

		default:

			//--  User entered wrong name of model
			help();
			return -1;
		}

		return 0;
	}

	return 0;
}
